---
jupytext:
  formats: ipynb,md:myst
  text_representation:
    extension: .md
    format_name: myst
    format_version: 0.13
    jupytext_version: 1.14.1
kernelspec:
  display_name: Python 3 (ipykernel)
  language: python
  name: python3
---

### Varianz etc. von Schulnoten


Diese Seite: Vorübungen und Hinführung zum Notebook {doc}`e_r1b`

Gegeben: Eine Liste von Schulnoten
* z.B. `Noten = [1, 2, 4, 2, 1]`

Gesucht:
* `Noten_avg`: das arithmetisches Mittel („Durchschnitt“) 
* `Noten_zentriert`: Subtrahiere von jeder Note den Durchschnitt
* `Noten_Varianz`: Bestimme die <https://de.wikipedia.org/wiki/Empirische_Varianz>.
* `Noten_standardisiert`: Dividiere jeden Wert aus `Noten_zentriert` durch die Standardabweichung (Wurzel der Varianz).

```{code-cell} ipython3
Noten = [1, 2, 4, 2, 1] # Schnitt: Kopfrechnen?
n = len(Noten)
```

Arithmetisches Mittel („Durchschnitt“): Summe dividiert durch Anzahl.

```{code-cell} ipython3
Noten_sum = 0
for x in Noten:
    Noten_sum += x
Noten_sum
```

```{code-cell} ipython3
Noten_avg = Noten_sum / n
Noten_avg
```

Messwerte so „verschieben“, dass der Durchschnitt auf der y-Achse (also `avg == 0`) liegt

```{code-cell} ipython3
Noten_zentriert = []

for x in Noten:
    Noten_zentriert.append( x - Noten_avg )

Noten_zentriert
```

oder als List Comprehension:

```{code-cell} ipython3
Noten_zentriert = [ x - Noten_avg for x in Noten ]
Noten_zentriert
```

Varianz: Summe der Quadrate der Abweichungen vom empirischen Wert zum Durchschnittswert

```{code-cell} ipython3
Noten_Varianz_1 = sum( [ (x - Noten_avg)**2  for x in Noten ] ) / n
Noten_Varianz_1
```

Die Varianz vom ursprünglichen und vom zentrierten Datensatz ist gleich, aber letztere berechnet sich noch einfacher:

```{code-cell} ipython3
Noten_Varianz = sum( [ x**2  for x in Noten_zentriert ] ) / n
Noten_Varianz
```

Standardabweichung = Quadratwurzel der Varianz

```{code-cell} ipython3
Noten_Standardabweichung = Noten_Varianz ** 0.5
Noten_Standardabweichung
```

Standardisiert = zentriert durch Standardabweichung

```{code-cell} ipython3
Noten_standardisiert = [ x / Noten_Standardabweichung for x in Noten_zentriert ]
Noten_standardisiert
```

## Wiederholung von Listenoperatoren: inplace oder Kopie?

```{code-cell} ipython3
def list_equality(x, y):
    """Vergleicht zwei Listen auf verschiedene Art und Weise."""
    
    print(f"{x=}\n{y=}\n{x == y =}\n{x is y =}")
    # print("x=", x, "y=", y, "Gleichheit", x == y, "Pointer Identität:", x is y)
```

```{code-cell} ipython3
l = [1, 2, 5, 3]
l1 = l

list_equality(l, l1)
```

```{code-cell} ipython3
l = [1, 2, 5, 3]
l2 = l

l2 += [] # leere Liste anhängen
# l2.append( [] ) # Äquivalent zu +=

list_equality(l, l2)
```

```{code-cell} ipython3
l = [1, 2, 5, 3]
l3 = l

l3 = l3 + []

list_equality(l, l3)
```

## in Funktionen einpacken

```{code-cell} ipython3
def avg(x):
    """Gibt das arithmetische Mittel der Elemente der Liste x zurück."""
    Summe = 0
    Anzahl = 0
    for i in x:
        Summe += i
        Anzahl += 1
    return Summe / Anzahl
```

oder kürzer, wenn man `sum()` und `len()` benutzen darf:

```{code-cell} ipython3
def avg2(x):
    return sum(x) / len(x)
```

oder noch kürzer:

```{code-cell} ipython3
avg3 = lambda x: sum(x) / len(x)
```

```{code-cell} ipython3
avg(Noten), avg2(Noten), avg3(Noten)
```

```{code-cell} ipython3
def zentriere(x):
    """Subtrahiert den avg(x) von jedem Element einer Liste x."""
    a = avg(x)
    return [ val - a for val in x ]
```

```{code-cell} ipython3
def zentriere_inplace(x):
    """Subtrahiert inplace den avg(x) von jedem Element einer Liste x."""
    a = avg(x)
    for i in range(len(x)):
        x[ i ] -= a 
```

```{code-cell} ipython3
l4 = l
l4 = zentriere(l)
list_equality(l, l4)
```

```{code-cell} ipython3
l5 = l
zentriere_inplace(l)
list_equality(l, l5)
```

```{code-cell} ipython3

```

```{code-cell} ipython3

```
